@@ -1,5 +1,9 @@
+import gc
 import operator
 import sys
+import weakref
+from collections import deque
+from collections.abc import Mapping
 from functools import reduce
 
 import pytest
@@ -67,6 +71,29 @@ def test_proxy_copy(dict_cls, proxy_cls):
     assert d1 is not d2
 
 
+@pytest.mark.skipif(
+    sys.version_info < (3, 7),
+    reason="__class_getitem__ is supported started from Python 3.7",
+)
+@pytest.mark.parametrize(
+    "cls",
+    ["MultiDict", "CIMultiDict", "MultiDictProxy", "CIMultiDictProxy"],
+    indirect=True,
+)
+def test_class_getitem(cls):
+    assert cls[str] is cls
+
+
+@pytest.mark.parametrize(
+    "cls",
+    ["MultiDict", "CIMultiDict", "MultiDictProxy", "CIMultiDictProxy"],
+    indirect=True,
+)
+def test_subclassing(cls):
+    class MyClass(cls):
+        pass
+
+
 class BaseMultiDictTest:
     def test_instantiate__empty(self, cls):
         d = cls()
@@ -108,6 +135,26 @@ class BaseMultiDictTest:
         assert sorted(d.values()) == [0, 1]
         assert sorted(d.items()) == [("0", 0), ("1", 1)]
 
+    def test_instantiate__from_list_of_lists(self, cls):
+        d = cls([["key", "value1"]])
+        assert d == {"key": "value1"}
+
+    def test_instantiate__from_list_of_custom_pairs(self, cls):
+        class Pair:
+            def __len__(self):
+                return 2
+
+            def __getitem__(self, pos):
+                if pos == 0:
+                    return "key"
+                elif pos == 1:
+                    return "value1"
+                else:
+                    raise IndexError
+
+        d = cls([Pair()])
+        assert d == {"key": "value1"}
+
     def test_getone(self, cls):
         d = cls([("key", "value1")], key="value2")
 
@@ -123,7 +170,8 @@ class BaseMultiDictTest:
         assert d.getone("key2", "default") == "default"
 
     def test__iter__(
-        self, cls,
+        self,
+        cls,
     ):
         d = cls([("key", "one"), ("key2", "two"), ("key", 3)])
         assert list(d) == ["key", "key2", "key"]
@@ -217,6 +265,38 @@ class BaseMultiDictTest:
 
         assert d1 != d2
 
+    def test_eq_bad_mapping_len(self, cls):
+        class BadMapping(Mapping):
+            def __getitem__(self, key):
+                return 1
+
+            def __iter__(self):
+                yield "a"
+
+            def __len__(self):
+                1 / 0
+
+        d1 = cls(a=1)
+        d2 = BadMapping()
+        with pytest.raises(ZeroDivisionError):
+            d1 == d2
+
+    def test_eq_bad_mapping_getitem(self, cls):
+        class BadMapping(Mapping):
+            def __getitem__(self, key):
+                1 / 0
+
+            def __iter__(self):
+                yield "a"
+
+            def __len__(self):
+                return 1
+
+        d1 = cls(a=1)
+        d2 = BadMapping()
+        with pytest.raises(ZeroDivisionError):
+            d1 == d2
+
     def test_ne(self, cls):
         d = cls([("key", "value1")])
 
@@ -294,6 +374,53 @@ class BaseMultiDictTest:
 
             assert sys.exc_info()[1] == e
 
+    def test_weakref(self, cls):
+        called = False
+
+        def cb(wr):
+            nonlocal called
+            called = True
+
+        d = cls()
+        wr = weakref.ref(d, cb)
+        del d
+        gc.collect()
+        assert called
+        del wr
+
+    def test_iter_length_hint_keys(self, cls):
+        md = cls(a=1, b=2)
+        it = iter(md.keys())
+        assert it.__length_hint__() == 2
+
+    def test_iter_length_hint_items(self, cls):
+        md = cls(a=1, b=2)
+        it = iter(md.items())
+        assert it.__length_hint__() == 2
+
+    def test_iter_length_hint_values(self, cls):
+        md = cls(a=1, b=2)
+        it = iter(md.values())
+        assert it.__length_hint__() == 2
+
+    def test_ctor_list_arg_and_kwds(self, cls):
+        arg = [("a", 1)]
+        obj = cls(arg, b=2)
+        assert list(obj.items()) == [("a", 1), ("b", 2)]
+        assert arg == [("a", 1)]
+
+    def test_ctor_tuple_arg_and_kwds(self, cls):
+        arg = (("a", 1),)
+        obj = cls(arg, b=2)
+        assert list(obj.items()) == [("a", 1), ("b", 2)]
+        assert arg == (("a", 1),)
+
+    def test_ctor_deque_arg_and_kwds(self, cls):
+        arg = deque([("a", 1)])
+        obj = cls(arg, b=2)
+        assert list(obj.items()) == [("a", 1), ("b", 2)]
+        assert arg == deque([("a", 1)])
+
 
 class TestMultiDict(BaseMultiDictTest):
     @pytest.fixture(params=["MultiDict", ("MultiDict", "MultiDictProxy")])
